/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.managed.factory.test;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.jboss.managed.api.ManagedObject;
import org.jboss.managed.api.ManagedProperty;
import org.jboss.managed.api.factory.ManagedObjectFactory;
import org.jboss.managed.plugins.factory.AbstractInstanceClassFactory;
import org.jboss.managed.plugins.factory.AbstractManagedObjectFactory;
import org.jboss.metatype.api.values.CollectionValue;
import org.jboss.metatype.api.values.GenericValue;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.test.managed.factory.AbstractManagedObjectFactoryTest;
import org.jboss.test.managed.factory.support.ManagementObjectChangedName;
import org.jboss.test.managed.factory.support.ManagementPropertySimpleManaged;
import org.jboss.test.managed.factory.support.Simple;
import org.jboss.test.managed.factory.support.amof.CustomName;
import org.jboss.test.managed.factory.support.amof.ManagementObjectWithRuntimeRef;
import org.jboss.test.managed.factory.support.amof.ManagementObjectWithRuntimeRefICF;
import org.jboss.test.managed.factory.support.amof.TestICF;
import org.jboss.test.managed.factory.support.amof.TestMOP;
import org.jboss.test.managed.factory.support.amof.TestSimpleICF;
import org.jboss.test.managed.factory.support.deployment.JmsDestination;
import org.jboss.test.managed.factory.support.deployment.JmsDestinationICF;
import org.jboss.test.managed.factory.support.deployment.JmsDestinations;
import org.jboss.test.managed.factory.support.deployment.QueueDestination;
import org.jboss.test.managed.factory.support.deployment.SecurityDomain;

/**
 * Test of behavior of the AbstractManagedObjectFactory
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 78207 $
 */
public class AbstractManagedObjectFactoryUnitTestCase extends AbstractManagedObjectFactoryTest
{
   private ManagedObjectFactory testMOF;

   public static Test suite()
   {
      return new TestSuite(AbstractManagedObjectFactoryUnitTestCase.class);
   }

   public AbstractManagedObjectFactoryUnitTestCase(String name)
   {
      super(name);
   }

   /**
    * Test overriding the AbstractManagedObjectFactory.defaultInstanceFactory 
    */
   public void testDefaultInstanceFactory()
   {
      AbstractManagedObjectFactory mof = new AbstractManagedObjectFactory();
      mof.setDefaultInstanceFactory(new TestICF());
      TestSimpleICF icf = new TestSimpleICF();
      mof.setInstanceClassFactory(Simple.class, icf);
      testMOF = mof;

      BigDecimal bigDecimal = new BigDecimal(10);
      Simple simple = new Simple();
      simple.setBigDecimalValue(bigDecimal);

      ManagedObject managedObject = initManagedObject(simple);
      checkManagedObjectDefaults(Simple.class, managedObject);
      checkDefaultManagedProperties(managedObject, Simple.class);

      checkPropertyDefaults(managedObject, "bigDecimalValue", BigDecimal.class, bigDecimal);
      assertTrue("TestBigDecimalICF.isGetValueCalled", icf.isGetValueCalled());

      //
      managedObject = super.createManagedObject(ManagementPropertySimpleManaged.class);
      checkManagedObjectDefaults(ManagementPropertySimpleManaged.class, managedObject);
      checkDefaultManagedProperties(managedObject, ManagementPropertySimpleManaged.class);
   }

   public void testDefaultManagedObjectPopulator()
   {
      AbstractManagedObjectFactory mof = new AbstractManagedObjectFactory();
      TestMOP mop = new TestMOP(mof.getConfiguration(), new TestSimpleICF(), mof.getInstanceFactories());
      mof.setDefaultManagedObjectPopulator(mop);
      testMOF = mof;

      BigDecimal bigDecimal = new BigDecimal(10);
      Simple simple = new Simple();
      simple.setBigDecimalValue(bigDecimal);

      ManagedObject managedObject = initManagedObject(simple);
      checkManagedObjectDefaults(Simple.class, managedObject);
      checkDefaultManagedProperties(managedObject, Simple.class);
      checkPropertyDefaults(managedObject, "bigDecimalValue", BigDecimal.class, bigDecimal);
      assertTrue("isPopulateManagedObjectCalled", mop.isPopulateManagedObjectCalled());
      //
      managedObject = createManagedObject(ManagementPropertySimpleManaged.class);
      checkManagedObjectDefaults(ManagementPropertySimpleManaged.class, managedObject);
      checkDefaultManagedProperties(managedObject, ManagementPropertySimpleManaged.class);
      assertTrue("isCreateObjectCalled", mop.isCreateObjectCalled());
 
   }

   @SuppressWarnings("unchecked")
   public void testGenericValueUpdate()
   {
      AbstractManagedObjectFactory mof = new AbstractManagedObjectFactory();
      mof.setInstanceClassFactory(JmsDestination.class, new JmsDestinationICF());
      testMOF = mof;

      JmsDestinations destinations = new JmsDestinations();
      List<JmsDestination> queues = new ArrayList<JmsDestination>();
      QueueDestination q1 = new QueueDestination();
      q1.setJndiName("queues/Q1");
      q1.setDomain(new SecurityDomain("java:/jaas/JMS"));
      queues.add(q1);
      destinations.setDestinations(queues);

      ManagedObject managedObject = initManagedObject(destinations);
      checkManagedObjectDefaults(JmsDestinations.class, managedObject);

      ManagedProperty destinationsMP = managedObject.getProperty("destinations");
      assertNotNull(destinationsMP);
      CollectionValue destinationsValue = (CollectionValue) destinationsMP.getValue();
      assertNotNull(destinationsValue);
      assertEquals(1, destinationsValue.getSize());
      GenericValue q1GV = (GenericValue) destinationsValue.getElements()[0];
      assertNotNull(q1GV);

      ManagedObject q1MO = (ManagedObject) q1GV.getValue();
      assertNotNull(q1MO);
      ManagedProperty domain = q1MO.getProperty("domain");
      getLog().info(domain);
      GenericValue domainGV = (GenericValue) domain.getValue();
      assertNotNull(domainGV);
      ManagedObject domainMO = (ManagedObject) domainGV.getValue();
      assertNotNull(domainMO);

      ManagedProperty domainName = domainMO.getProperty("domain");
      assertEquals(SimpleValueSupport.wrap("java:/jaas/JMS"), domainName.getValue());

      ManagedObject domainUpdateMO = initManagedObject(new SecurityDomain("java:/jaas/JMS2"));
      AbstractInstanceClassFactory icf = new AbstractInstanceClassFactory(mof);
      GenericValue domainUpdateGV = icf.getManagedObjectValue(domain, domainUpdateMO);
      assertNotNull(domainUpdateGV.getValue());
      domain.setValue(domainUpdateGV);
      // Validate this has been written through
      assertEquals("java:/jaas/JMS2", q1.getDomain().getDomain());
   }

   /**
    * Test the transformer value of the ManagementRuntimeRef annotation when
    * there is a custom ICF
    */
   public void testTransformer()
   {
      AbstractManagedObjectFactory mof = new AbstractManagedObjectFactory();
      mof.setInstanceClassFactory(ManagementObjectWithRuntimeRef.class, new ManagementObjectWithRuntimeRefICF());
      testMOF = mof;

      ManagementObjectWithRuntimeRef mowref = new ManagementObjectWithRuntimeRef();
      mowref.setRefName(new CustomName("jboss:test=testTransformer"));
      ManagedObject managedObject = initManagedObject(mowref);
      createManagedObject(ManagementObjectChangedName.class);
      assertEquals("jboss:test=testTransformer", managedObject.getComponentName());
   }

   /**
    * Override to allow test specific ManagedObjectFactory
    */
   @Override
   protected ManagedObjectFactory getMOF()
   {
      ManagedObjectFactory mof = testMOF;
      if (mof == null)
         mof = super.getMOF();
      return mof;
   }

}
